import React from 'react'
import { routes } from './routes'
import { Layout, Menu } from 'antd'
import { Routes, Route, Link, Navigate } from 'react-router-dom'
import styled from 'styled-components'
const { Content, Sider } = Layout

const SidebarDiv = styled.div`
  .sidebar {
    overflow: auto;
    height: 100vh;
    position: fixed;
    left: 0;
    top: 0;
    bottom: 0;
  }
  .content {
    overflow: initial;
    margin-left: 200px;
  }
  .headerLogo {
    height: 32px;
    margin: 16px;
    background: rgba(255, 255, 255, 0.2);
    text-align: center;
    color: #fff;
    line-height: 32px;
  }
`

const Main = () => {
  const menus = routes.map((item, index) => {
    return {
      key: String(index + 1),
      label: <Link to={item.path}>{item.title}</Link>,
    }
  })
  const openJupyterlab = () => {
    window.open(process.env.REACT_APP_JPT, '_target')
  }
  menus.push({
    key: 'jupyterlab',
    label: <div onClick={openJupyterlab}>可视化编程</div>,
  })

  let childrenList = []
  const routesInfo = routes.map(item => {
    if (item.children.length > 0) {
      childrenList = childrenList.concat(...item.children)
    }
    return {
      path: item.path,
      exact: item?.exact,
      component: item.component,
    }
  })
  const list = routesInfo.concat(...childrenList)

  return (
    <SidebarDiv>
      <Layout hasSider>
        <Sider className="sidebar">
          <div className="headerLogo">AI平台</div>
          <Menu theme="dark" mode="inline" items={menus} />
        </Sider>
        <Content className="content">
          <Routes>
            {list.map((route, index) => (
              <Route
                key={index}
                path={route.path}
                exact={route?.exact}
                element={<route.component />}
              />
            ))}
            <Route path="/" element={<Navigate to="datasource" replace />} />
          </Routes>
        </Content>
      </Layout>
    </SidebarDiv>
  )
}

export default Main
