import { Input, Radio, Select, Space } from 'antd'
import React, {
  useState,
  useImperativeHandle,
  forwardRef,
  useEffect,
} from 'react'
import styled from 'styled-components'

const CronWrapper = styled.div`
  .time_wrapper {
    margin-top: 10px;
  }
  .time_select {
    width: 58px;
  }
  .cron_input {
    width: 60%;
    margin-right: 10px;
  }
  .cron_format {
    display: block;
    margin: 10px 0;
  }
`

const RadioGroup = Radio.Group

const { Option } = Select

const CronSelect = ({ cron_data }, ref) => {
  const [cronType, setCronType] = useState('0')
  const [hourType, setHourType] = useState('hour')
  const [hourMax, setHourMax] = useState(24)
  const [minVal, setMinVal] = useState('')
  const [hourVal, setHourVal] = useState('')
  const [dayVal, setDayVal] = useState('')
  const [weekVal, setWeekVal] = useState('')
  const [monthVal, setMonthVal] = useState('')
  const [cronVal, setCronVal] = useState('')
  const [showExample, setShowExample] = useState(false)
  useImperativeHandle(ref, () => {
    return {
      getCronData: formatCronData(),
    }
  })

  useEffect(() => {
    const cron_type = cron_data?.cron_select_type
    let keys = []
    if (![null, undefined].includes(cron_type)) keys = Object.keys(cron_data)
    setCronType(String(cron_type))
    switch (cron_type) {
      case 0:
        if (keys.includes('hour')) {
          setHourVal(cron_data['hour'])
          onChangeHourType('hour')
        } else {
          setHourVal(cron_data['minute'])
          onChangeHourType('min')
        }
        break
      case 1:
        setHourVal(cron_data['hour'])
        setMinVal(cron_data['minute'])
        break
      case 2:
        setWeekVal(cron_data['week'])
        setHourVal(cron_data['hour'])
        setMinVal(cron_data['minute'])
        break
      case 3:
        setMonthVal(cron_data['month'])
        setDayVal(cron_data['day'])
        setHourVal(cron_data['hour'])
        setMinVal(cron_data['minute'])
        break
      case 4:
        setCronVal(cron_data['cron_expression'])
        break
      default:
        break
    }
  }, [cron_data])

  const formatCronData = () => {
    let obj = {}
    switch (cronType) {
      case '0':
        if (hourType === 'hour') {
          obj = { cron_select_type: Number(cronType), hour: hourVal }
        } else {
          obj = { cron_select_type: Number(cronType), minute: hourVal }
        }
        break
      case '1':
        obj = {
          cron_select_type: Number(cronType),
          hour: hourVal,
          minute: minVal,
        }
        break
      case '2':
        obj = {
          cron_select_type: Number(cronType),
          week: weekVal,
          hour: hourVal,
          minute: minVal,
        }
        break
      case '3':
        obj = {
          cron_select_type: Number(cronType),
          month: monthVal,
          day: dayVal,
          hour: hourVal,
          minute: minVal,
        }
        break
      case '4':
        obj = { cron_select_type: Number(cronType), cron_expression: cronVal }
        break
      default:
        break
    }
    return obj
  }

  const onCronTypeChange = e => {
    setCronType(e.target.value)
    setMinVal('')
    setHourVal('')
    setDayVal('')
    setWeekVal('')
    setMonthVal('')
  }
  const onChangeHourType = val => {
    setHourType(val)
    setHourMax(val === 'hour' ? 24 : 60)
  }

  const onChangeHourVal = val => {
    setHourVal(val)
  }
  const onChangeDayVal = val => {
    setDayVal(val)
  }
  const onChangeWeekVal = val => {
    setWeekVal(val)
  }
  const onChangeMonthVal = val => {
    setMonthVal(val)
  }
  const onChangeMinVal = val => {
    setMinVal(val)
  }
  const onChangeCronVal = e => {
    setCronVal(e.target.value)
  }
  const onCronExampleChange = val => {
    setCronVal(val)
  }

  // 各时间段选项
  const timeOptions = (start, val) => {
    const options = []
    for (let i = start; i < val; i++) {
      options.push(
        <Option value={i} key={i}>
          {i}
        </Option>
      )
    }
    return options
  }

  const hourSet = (
    <Space>
      <span>每隔</span>
      <Select
        onChange={onChangeHourVal}
        value={hourVal}
        className="time_select">
        {timeOptions(1, hourMax)}
      </Select>
      <Select onChange={onChangeHourType} value={hourType}>
        <Option value="min">分钟</Option>
        <Option value="hour">小时</Option>
      </Select>
      <span>执行一次</span>
    </Space>
  )

  const daySet = (
    <Space>
      <span>每天</span>
      <Select
        onChange={onChangeHourVal}
        value={hourVal}
        className="time_select">
        {timeOptions(0, 24)}
      </Select>
      <span>时</span>
      <Select onChange={onChangeMinVal} value={minVal} className="time_select">
        {timeOptions(0, 60)}
      </Select>
      <span>分</span>
      <span>执行一次</span>
    </Space>
  )

  const weekSet = (
    <Space>
      <span>每周</span>
      <Select
        onChange={onChangeWeekVal}
        value={weekVal}
        className="time_select">
        <Option value="1">一</Option>
        <Option value="2">二</Option>
        <Option value="3">三</Option>
        <Option value="4">四</Option>
        <Option value="5">五</Option>
        <Option value="6">六</Option>
        <Option value="7">七</Option>
      </Select>
      <span>，</span>
      <Select
        onChange={onChangeHourVal}
        value={hourVal}
        className="time_select">
        {timeOptions(0, 24)}
      </Select>
      <span>时</span>
      <Select onChange={onChangeMinVal} value={minVal} className="time_select">
        {timeOptions(0, 60)}
      </Select>
      <span>分</span>
      <span>执行一次</span>
    </Space>
  )

  const monthSet = (
    <Space>
      <span>每隔</span>
      <Select
        onChange={onChangeMonthVal}
        value={monthVal}
        className="time_select">
        {timeOptions(1, 13)}
      </Select>
      <span>个月，</span>
      <Select onChange={onChangeDayVal} value={dayVal} className="time_select">
        {timeOptions(1, 32)}
      </Select>
      <span>日，</span>
      <Select
        onChange={onChangeHourVal}
        value={hourVal}
        className="time_select">
        {timeOptions(0, 24)}
      </Select>
      <span>时</span>
      <Select onChange={onChangeMinVal} value={minVal} className="time_select">
        {timeOptions(0, 60)}
      </Select>
      <span>分</span>
      <span>执行一次</span>
    </Space>
  )

  const cronExample = (
    <Select allowClear onChange={onCronExampleChange} placeholder="查看示例">
      <Option value="0 * * * *">
        <span>0 * * * * (每小时)</span>
      </Option>
      <Option value="0 0 * * 1">
        <span>0 0 * * 1(每周)</span>
      </Option>
      <Option value="0 0 1 * *">
        <span>0 0 1 * *(每月)</span>
      </Option>
      <Option value="*/15 * * * *">
        <span>*/15 * * * *(每隔15分钟执行一次)</span>
      </Option>
      <Option value="* */12 * * *">
        <span>* */12 * * *(每隔12小时执行一次)</span>
      </Option>
      <Option value="15 10 16 * *">
        <span>15 10 16 * *(每月16日10:15分运行)</span>
      </Option>
    </Select>
  )

  const cronSet = (
    <div>
      <Input
        className="cron_input"
        placeholder="* * * * *"
        value={cronVal}
        onChange={onChangeCronVal}
      />
      {!showExample && <span onClick={() => setShowExample(true)}>示例</span>}
      {showExample && <span onClick={() => setShowExample(false)}>隐藏</span>}
      <div>
        <span className="cron_format">格式：分 时 天 月 周</span>
        {showExample && cronExample}
      </div>
    </div>
  )

  return (
    <CronWrapper>
      <RadioGroup onChange={onCronTypeChange} value={cronType}>
        <Radio value="0">按小时</Radio>
        <Radio value="1">按天</Radio>
        <Radio value="2">按星期</Radio>
        <Radio value="3">按月</Radio>
        <Radio value="4">Cron表达式</Radio>
      </RadioGroup>
      <div
        className="time_wrapper"
        onClick={e => {
          e.preventDefault()
          e.stopPropagation()
        }}>
        {cronType === '0' && hourSet}
        {cronType === '1' && daySet}
        {cronType === '2' && weekSet}
        {cronType === '3' && monthSet}
        {cronType === '4' && cronSet}
      </div>
    </CronWrapper>
  )
}

export default forwardRef(CronSelect)
