import React, { useEffect, useRef, useState } from 'react'
import StepFour from './StepFour'
import DataTableStruct from './DataTableStruct'
import { useNavigate, useLocation } from 'react-router-dom'
import styled from 'styled-components'
import { Button, Form, Space, Drawer, message } from 'antd'
import { MenuFoldOutlined, MenuUnfoldOutlined } from '@ant-design/icons'
import { getJobInfo, updateJobList } from '../services'
import moment from 'moment'
const EditView = styled.div`
  padding: 20px;
  .check__btn {
    position: absolute;
    right: 0;
    top: 20%;
    border-radius: 0;
  }
`

const EditSyncView = () => {
  // 路由导航
  const navigate = useNavigate()
  const { state } = useLocation()

  const cronRef = useRef()
  // jsonValue
  const jsonValue = React.createRef()

  // 同步参数表单
  const [syncDataForm] = Form.useForm()
  const [partitionNum, setPartitionNum] = useState(0)
  // 右侧表结构可视
  const [visible, setVisible] = useState(false)
  const [dataxJson, setDataxJson] = useState(null)
  const [cron_data, setCronData] = useState(null)
  const [trigger_status, setTriggerStatus] = useState(false)

  const fetchSyncInfo = async id => {
    const { data } = await getJobInfo(id)
    if (data.code === 200) {
      syncDataForm.setFieldsValue({
        job_desc: data.data.job_desc,
        executor_route_strategy: data.data.executor_route_strategy,
        executor_block_strategy: data.data.executor_block_strategy,
        cron_expression: data.data.cron_expression_dict,
        executor_timeout: data.data.executor_timeout,
        executor_fail_retry_count: data.data.executor_fail_retry_count,
        inc_start_time:
          data.data.inc_start_time && moment(data.data.inc_start_time * 1000),
        last_time: data.data.last_time,
        current_time: data.data.current_time,
        partition_info: data.data.partition_info,
        partition_time: data.data.partition_time,
        jvm_param: data.data.jvm_param,
      })
      setTriggerStatus(data.data.trigger_status)
      setCronData(data.data.cron_expression_dict)
      setPartitionNum(data.data.partition_num)
      setDataxJson(JSON.parse(data.data.job_json))
    } else {
      message.error(data.msg)
    }
  }
  const submit = () => {
    if (jsonValue.current) {
      setDataxJson(JSON.parse(jsonValue.current.jsonValue.editor.getValue()))
    }
    try {
      let job_json = JSON.stringify(dataxJson)
      if (jsonValue.current) {
        job_json = JSON.stringify(
          JSON.parse(jsonValue.current.jsonValue.editor.getValue())
        )
      }
      if (Object.keys(cronRef.current.getCronData).length !== 0) {
        syncDataForm.setFieldValue(
          'cron_expression',
          cronRef.current.getCronData
        )
      }
      syncDataForm
        .validateFields()
        .then(() => {
          handleEdit(job_json)
        })
        .catch(err => {
          message.error('请检查表单数据是否完整')
        })
    } catch (error) {
      message.error('转换JSON字符串失败，请检查json数据', error)
    }
  }

  const handleEdit = async job_json => {
    const fields = syncDataForm.getFieldValue()
    const params = {
      ...fields,
      trigger_status,
      job_json,
      inc_start_time: moment(fields.inc_start_time).unix(),
      user_id: 'test',
    }
    params['partition_num'] = partitionNum
    const { data } = await updateJobList(state.id, params)
    if (data.code === 200) {
      message.success('提交成功')
      navigate(-1)
    } else {
      message.error('提交失败')
    }
  }

  // 展示
  const showDrawer = () => {
    setVisible(true)
  }

  // 关闭
  const onClose = () => {
    if (jsonValue.current) {
      setDataxJson(JSON.parse(jsonValue.current.jsonValue.editor.getValue()))
    }

    setVisible(false)
  }

  useEffect(() => {
    if (state) {
      fetchSyncInfo(state.id)
    }
  }, [state])
  return (
    <EditView>
      <StepFour
        onRef={cronRef}
        syncDataForm={syncDataForm}
        cron_data={cron_data}
        partitionNumHandle={{
          num: partitionNum,
          func: val => setPartitionNum(val),
        }}
      />
      <Space style={{ margin: '20px' }}>
        <Button
          onClick={() => {
            navigate(-1)
          }}>
          取消
        </Button>
        <Button type="primary" onClick={submit}>
          提交
        </Button>
      </Space>
      <Button
        type="primary"
        onClick={showDrawer}
        className="check__btn"
        size="large"
        icon={<MenuFoldOutlined />}>
        构建
      </Button>
      <Drawer
        title="构建"
        placement="right"
        onClose={onClose}
        visible={visible}
        width={600}
        mask={false}
        destroyOnClose={true}
        closeIcon={<MenuUnfoldOutlined style={{ color: '#1881DA' }} />}>
        <DataTableStruct
          currentStep={3}
          tableData={{}}
          datax={dataxJson}
          onRef={jsonValue}
        />
      </Drawer>
    </EditView>
  )
}
export default EditSyncView
