import { Tabs, Button } from 'antd'
import DataSourceManage from '../component/DatasourceManage.jsx'
import DatasourceAdd from '../component/DatasourceAdd'
import DatasourceSyncView from '../component/DatasourceSyncView'
import DatasourceLog from '../component/DatasourceLog'
import React, { useState } from 'react'
import { Link } from 'react-router-dom'
import styled from 'styled-components'
const { TabPane } = Tabs

const Datasource = styled.div`
  padding: 20px;
`

export default function DatasourceView() {
  // 数据源管理ref
  const sourceRef = React.createRef()
  const logRef = React.createRef()
  // 当前所处页面
  const [currentKey, setCurrentKey] = useState('1')
  const [currentLog, setCurrentLog] = useState(null)

  // 更新数据源列表
  const updateDataSource = () => {
    sourceRef.current.updateSourceList()
  }
  const updateLogs = () => {
    logRef.current?.updateLogList()
  }

  const selectLog = id => {
    setCurrentLog(id)
    setCurrentKey('3')
  }
  // Tab标签右边按钮
  const tabExtraBtn = () => {
    switch (currentKey) {
      case '1':
        return <DatasourceAdd updateDataSource={updateDataSource} />
      case '2':
        return (
          <Link to="/datasource/sync-task">
            <Button type="primary">添加同步任务</Button>
          </Link>
        )
      default:
        return
    }
  }
  return (
    <Datasource>
      {/* Tab栏及内容 */}
      <Tabs
        type="card"
        tabBarExtraContent={tabExtraBtn()}
        activeKey={currentKey}
        onTabClick={key => {
          if (key === '3') updateLogs()
        }}
        onChange={key => {
          setCurrentKey(key)
          setCurrentLog(null)
        }}>
        <TabPane tab="数据源管理" key="1">
          <DataSourceManage onRef={sourceRef} />
        </TabPane>
        <TabPane tab="同步配置" key="2">
          <DatasourceSyncView selectLog={id => selectLog(id)} />
        </TabPane>
        <TabPane tab="日志管理" key="3">
          <DatasourceLog onRef={logRef} logId={currentLog} />
        </TabPane>
      </Tabs>
    </Datasource>
  )
}
