import { Card, message, Table } from 'antd'
import React, { useEffect, useState } from 'react'
import { useLocation } from 'react-router-dom'
import styled from 'styled-components'
import { getAllTask, getTaskLog } from '../services/index'
import moment from 'moment'

const LogWrapper = styled.div`
  padding: 20px;
  display: flex;
  height: 100vh;
  column-gap: 20px;
  .job-list {
    flex: 1;
  }
  .log-info {
    width: 920px;
  }
  .log {
    background-color: #012b36;
    height: 100%;
    width: 100%;
    overflow-y: scroll;
    color: #638691;
    padding: 8px 10px;
    ::-webkit-scrollbar {
      display: none;
    }
  }

  .node_list {
    width: 100%;
    height: 100%;
  }
  .node_table {
    overflow: auto;
    height: 650px
    ::-webkit-scrollbar {
      display: none;
    }
  }
  .ant-table-selection-column {
    display: none !important;
  }
`

let timeout = null

const TaskLogWatcher = () => {
  const { state } = useLocation()
  const [logData, setLogData] = useState('暂无日志')
  // 初始化节点列表
  const [jobList, setJobList] = useState([])
  // 选中的作业
  const [selectJob, setSelectJob] = useState(null)

  const columns = [
    {
      title: '序号',
      dataIndex: 'index',
      key: 'index',
    },
    {
      title: '作业名称',
      dataIndex: 'name',
      key: 'name',
    },
    {
      title: '执行时间',
      dataIndex: 'executeTime',
      key: 'executeTime',
    },
    {
      title: '执行结果',
      dataIndex: 'handleResult',
      key: 'handleResult',
      render: code => (
        <span
          style={{
            color: code === 3 ? '#FF4D4F' : code === 2 ? '#52C41A' : '#4A4A4A',
          }}>
          {code === 3
            ? '失败'
            : code === 2
            ? '成功'
            : code === 1
            ? '运行中'
            : '队列中'}
        </span>
      ),
    },
  ]

  const fetchAllTask = async () => {
    const { data } = await getAllTask(state.id)
    if (data.code === 200) {
      formatSingleTask(data.data)
    } else {
      message.error(data.msg)
    }
  }

  const formatSingleTask = data => {
    const list = data.map((item, index) => ({
      key: index,
      id: item.id,
      index: index + 1,
      executeTime: moment(Math.round(item.execute_time) * 1000).format(
        'YYYY.MM.DD HH:mm:ss'
      ),
      name: item.name,
      log: item.log,
      handleResult: item.execute_result,
      task_id: item.task_id,
      job_id: item.job_id,
      af_run_id: item.af_run_id,
    }))
    setJobList(list)
    onClickTableNode(list[0])
  }

  const onClickTableNode = async record => {
    fetchTaskLog({
      job_id: record.job_id,
      task_id: record.task_id,
      af_run_id: record.af_run_id,
    })
    setSelectJob(record.key)
  }

  const fetchTaskLog = async params => {
    const { data } = await getTaskLog(params)
    if (data.code === 200) {
      setLogData(data.data.log)
      if (!['success', 'failed'].includes(data.data.status)) {
        timeout = setTimeout(() => {
          fetchTaskLog(params)
        }, 3000)
      }
    } else {
      message.error(data.msg)
    }
  }

  useEffect(() => {
    fetchAllTask()
  }, [state])

  useEffect(() => {
    return () => {
      clearTimeout(timeout)
    }
  }, [])

  const singleTable = (
    <Table
      columns={columns}
      dataSource={jobList}
      showHeader={false}
      className="node_table"
      pagination={{ position: ['none', 'none'] }}
      onRow={record => {
        return {
          onClick: () => onClickTableNode(record),
        }
      }}
      rowSelection={{
        type: 'radio',
        columnWidth: '0',
        selectedRowKeys: [selectJob],
      }}
    />
  )
  return (
    <LogWrapper>
      {/* <pre className="log">{logData}</pre> */}
      <div className="job-list">
        <Card
          size="small"
          title="节点列表"
          bordered={false}
          className="node_list"
          headStyle={{
            padding: '10px 15px',
            fontSize: '14px',
            fontWeight: '700',
          }}>
          {/* {taskType === '单作业离线任务' ? singleTable : multiTree} */}
          {singleTable}
        </Card>
      </div>
      <div className="log-info">
        <Card
          bordered={false}
          style={{ height: '100%', width: '100%' }}
          bodyStyle={{ height: '100%', width: '100%' }}>
          <pre className="log">{logData}</pre>
        </Card>
      </div>
    </LogWrapper>
  )
}

export default TaskLogWatcher
